/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.usercontrols;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

import ca.uvic.csr.shrimp.FilterConstants;
import ca.uvic.csr.shrimp.SelectorBeanConstants;
import ca.uvic.csr.shrimp.ShrimpConstants;
import ca.uvic.csr.shrimp.DataBean.Relationship;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpArc;
import ca.uvic.csr.shrimp.FilterBean.Filter;
import ca.uvic.csr.shrimp.FilterBean.FilterBean;
import ca.uvic.csr.shrimp.FilterBean.FilterNotFoundException;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;

/**
 * This adapter handles requests to filter the currently selected arcs.
 * An id filter is added to the FilterBean for each selected relationship.
 *
 * @tag Shrimp.filterAllArcs
 * @author Chris Callendar
 * @date Feb 6, 2007
 */
public class FilterSelectedArcsAdapter extends DefaultToolAction {
	
	public static final String ACTION_NAME = ShrimpConstants.ACTION_NAME_HIDE_ARC;
	public static final String TOOLTIP = "Hides the selected arc.";
	
	private ShrimpArcIDFilter filter;
	
	/**
	 * Constructs a new FilterSelectedArcsAdapter
	 * @param tool The tool that this adapter acts upon.
	 */
	public FilterSelectedArcsAdapter(ShrimpTool tool) {
		super(ACTION_NAME, tool);
		setToolTip(TOOLTIP);
		this.mustStartAndStop = false;
		this.filter = new ShrimpArcIDFilter();
	}
	
	public void startAction() {
	    try {
			FilterBean filterBean = (FilterBean) tool.getBean(ShrimpTool.DISPLAY_FILTER_BEAN);		
			SelectorBean selectorBean = (SelectorBean) tool.getBean(ShrimpTool.SELECTOR_BEAN);
	    	Vector selectedArcs = (Vector) selectorBean.getSelected(SelectorBeanConstants.SELECTED_ARCS);
			Vector idsToFilter = new Vector (selectedArcs.size());
			for (Iterator iterator = selectedArcs.iterator(); iterator.hasNext();) {
				ShrimpArc arc = (ShrimpArc) iterator.next();
				Relationship rel = arc.getRelationship();
				if (rel != null) {
					idsToFilter.add(new Long(rel.getID()));
				}
			}
			if (!idsToFilter.isEmpty()) {
				filter.addIDs(idsToFilter);
				if (filterBean.hasFilter(filter)) {
					try {
						filterBean.removeFilter(filter);
					} catch (FilterNotFoundException e) {}
				}
				filterBean.addFilter(filter);
			}
		} catch (BeanNotFoundException stnfe) {
			stnfe.printStackTrace();
	    }
	}
	
	public boolean isEnabled() {
		try {
			SelectorBean selectorBean = (SelectorBean) tool.getBean(ShrimpTool.SELECTOR_BEAN);
	    	Vector selectedArcs = (Vector) selectorBean.getSelected(SelectorBeanConstants.SELECTED_ARCS);
			return super.isEnabled() && (selectedArcs.size() > 0);
		} catch (BeanNotFoundException stnfe) {
			return false;
		}
	}
	
	/**
	 * Removes the filter and clears the ids.
	 */
	public void clearFilter() {
		 try {
			filter.clearIDs();
			FilterBean filterBean = (FilterBean) tool.getBean(ShrimpTool.DISPLAY_FILTER_BEAN);		
			if (filterBean.hasFilter(filter)) {
				try {
					filterBean.removeFilter(filter);
				} catch (FilterNotFoundException e) {}
			}
		} catch (BeanNotFoundException stnfe) {
			stnfe.printStackTrace();
	    }
	}
	
	/**
	 * @return true if the filter has at least one relationship id to hide. 
	 */
	public boolean hasFilteredIDs() {
		return (filter.getSize() > 0);
	}
	
	/**
	 * A filter which hides {@link Relationship}s by id. 
	 * @author Chris Callendar
	 * @date 6-Feb-07
	 */
	class ShrimpArcIDFilter implements Filter {
		
		private Collection/*<Long>*/ ids = new HashSet();
		
		public void addIDs(Collection/*<Long>*/ idsToAdd) {
			this.ids.addAll(idsToAdd);
		}
		
		public void clearIDs() {
			ids.clear();
		}
		
		public int getSize() {
			return ids.size();
		}
		
		public String toString() {
			return "Relationship ID Filter";
		}

		public String getFilterType() {
			return FilterConstants.RELATIONSHIP_ID_FILTER_TYPE;
		}

		public String getTargetType() {
			return FilterConstants.RELATIONSHIP_FILTER_TYPE;
		}

		public boolean isFiltered(Object object) {
			boolean filtered = false;
			if (object instanceof Relationship) {
				Relationship rel = (Relationship) object;
				Long id = new Long(rel.getID());
				if (ids.contains(id)) {
					filtered = true;
				}
			}
			return filtered;
		}
		
	}
			
}